/* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
 * Copyright 1998-2003 Decklin Foster <decklin@red-bean.com>
 * This program is free software; see LICENSE for details. */

#include <signal.h>
#include <X11/cursorfont.h>
#include "aewm.h"
#include "parser.h"

Display *dpy;
Window root;
int screen;
XFontStruct *font;
#ifdef XFT
XftFont *xftfont;
XftColor xft_fg;
#endif
GC invert_gc;
GC string_gc;
GC border_gc;
XColor fg;
XColor bg;
XColor bd;
Cursor move_curs;
Cursor resize_curs;
Atom wm_protos;
Atom wm_delete;
Atom wm_state;
Atom wm_change_state;
#ifdef GNOME_PDA
Atom gnome_pda;
#endif
#ifdef MWM_HINTS
Atom mwm_hints;
#endif
client_t *head_client;
#if 0
static char *opt_font = DEF_FONT;
static char *opt_fg = DEF_FG;
static char *opt_bg = DEF_BG;
static char *opt_bd = DEF_BD;
char *opt_new1 = DEF_NEW1;
char *opt_new2 = DEF_NEW2;
char *opt_new3 = DEF_NEW3;
int opt_bw = DEF_BW;
int opt_pad = DEF_PAD;
#endif
#ifdef SHAPE
Bool shape;
int shape_event;
#endif
int windows_wm_major_opcode, windows_wm_event, windows_wm_error;

static void scan_wins(void);
static void setup_display(void);

#define USAGE \
    "usage: aewm_win [--help|-h]\n" \
        "            [--version|-v]\n"

int main(int argc, char **argv)
{
    int i;
    struct sigaction act;

    for (i = 1; i < argc; i++) {
        if ARG("version", "v",0) {
            printf("aewm: version " VERSION "\n");
            exit(0);
        }
        if ARG("help", "h",0) {
            printf(USAGE);
            exit(0);
        }
        /* if we get here, it must be a bad option */
        err("unknown option: '%s'\n" USAGE, argv[i]);
        exit(2);
    }

    act.sa_handler = sig_handler;
    act.sa_flags = 0;
    sigaction(SIGTERM, &act, NULL);
    sigaction(SIGINT, &act, NULL);
    sigaction(SIGHUP, &act, NULL);
    sigaction(SIGCHLD, &act, NULL);

    setup_display();
    scan_wins();
    do_event_loop();
    return 1; /* just another brick in the -Wall */
}

static void scan_wins(void)
{
    unsigned int nwins, i;
    Window dummyw1, dummyw2, *wins;
    XWindowAttributes attr;

    XQueryTree(dpy, root, &dummyw1, &dummyw2, &wins, &nwins);
    for (i = 0; i < nwins; i++) {
        XGetWindowAttributes(dpy, wins[i], &attr);
        if (!attr.override_redirect && attr.map_state == IsViewable)
            make_new_client(wins[i]);
    }
    XFree(wins);
}

static void setup_display(void)
{
    XSetWindowAttributes sattr;
#ifdef SHAPE
    int dummy;
#endif

    dpy = XOpenDisplay(NULL);

    if (!dpy) {
        err("can't open display '%s' (is $DISPLAY set properly?)",
            getenv("DISPLAY"));
        exit(1);
    }

    XSetErrorHandler(handle_xerror);
    screen = DefaultScreen(dpy);
    root = RootWindow(dpy, screen);

    wm_protos = XInternAtom(dpy, "WM_PROTOCOLS", False);
    wm_delete = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
    wm_state = XInternAtom(dpy, "WM_STATE", False);
    wm_change_state = XInternAtom(dpy, "WM_CHANGE_STATE", False);
#ifdef GNOME_PDA
    gnome_pda = XInternAtom(dpy, "GNOME_PANEL_DESKTOP_AREA", False);
#endif
#ifdef MWM_HINTS
    mwm_hints = XInternAtom(dpy, _XA_MWM_HINTS, False);
#endif

#ifdef SHAPE
    shape = XShapeQueryExtension(dpy, &shape_event, &dummy);
#endif

    XQueryExtension(dpy, WINDOWSWMNAME, &windows_wm_major_opcode, &windows_wm_event, &windows_wm_error);
    XWindowsWMSelectInput(dpy, WindowsWMControllerNotifyMask | WindowsWMActivationNotifyMask);

    sattr.event_mask = ChildMask|ColormapChangeMask|ButtonMask;
    XChangeWindowAttributes(dpy, root, CWEventMask, &sattr);
}
